/// <reference types="node" />
import { Server as HttpServer } from 'http';
import { GraphQLSchema, GraphQLError, GraphQLResolveInfo, ValidationContext } from 'graphql';
import { LogFunction } from './runQuery';
import { Config, ListenOptions, RegistrationOptions, ServerInfo } from './types';
export declare class ApolloServerBase<Request = RequestInit> {
    subscriptions: Config<Request>['subscriptions'];
    disableTools: boolean;
    private schema;
    private context?;
    private requestOptions;
    private graphqlPath;
    private engine;
    private engineEnabled;
    private http?;
    protected getHttp: () => HttpServer;
    constructor(config: Config<Request>);
    use({getHttp, path}: RegistrationOptions): void;
    listen(opts?: ListenOptions): Promise<ServerInfo>;
    stop(): Promise<void>;
    private createSubscriptionServer(server, config);
    private createEngine({engineInRequestPath, engine});
    request(request: Request): Promise<{
        schema: GraphQLSchema;
        formatError: Function | ((e: GraphQLError) => GraphQLError);
        rootValue?: any;
        context: any;
        logFunction?: LogFunction;
        formatParams?: Function;
        validationRules?: ((context: ValidationContext) => any)[];
        formatResponse?: Function;
        fieldResolver?: (source: any, args: {
            [argName: string]: any;
        }, context: any, info: GraphQLResolveInfo) => any;
        debug?: boolean;
        tracing: boolean;
        cacheControl: any;
    }>;
}
